@@ -6,7 +6,7 @@ Discord API Wrapper
 
 A basic wrapper for the Discord API.
 
-:copyright: (c) 2015-2020 Rapptz
+:copyright: (c) 2015-present Rapptz
 :license: MIT, see LICENSE for more details.
 
 """
@@ -14,8 +14,8 @@ A basic wrapper for the Discord API.
 __title__ = 'discord'
 __author__ = 'Rapptz'
 __license__ = 'MIT'
-__copyright__ = 'Copyright 2015-2020 Rapptz'
-__version__ = '1.5.1'
+__copyright__ = 'Copyright 2015-present Rapptz'
+__version__ = '1.7.0'
 
 __path__ = __import__('pkgutil').extend_path(__path__, __name__)
 
@@ -33,12 +33,12 @@ from .guild import Guild
 from .flags import *
 from .relationship import Relationship
 from .member import Member, VoiceState
-from .message import Message, MessageReference, Attachment
+from .message import *
 from .asset import Asset
 from .errors import *
 from .calls import CallMessage, GroupCall
 from .permissions import Permissions, PermissionOverwrite
-from .role import Role
+from .role import Role, RoleTags
 from .file import File
 from .colour import Color, Colour
 from .integrations import Integration, IntegrationAccount
@@ -58,16 +58,10 @@ from .voice_client import VoiceClient, VoiceProtocol
 from .audit_logs import AuditLogChanges, AuditLogEntry, AuditLogDiff
 from .raw_models import *
 from .team import *
+from .sticker import Sticker
 
 VersionInfo = namedtuple('VersionInfo', 'major minor micro releaselevel serial')
 
-version_info = VersionInfo(major=1, minor=5, micro=1, releaselevel='final', serial=0)
+version_info = VersionInfo(major=1, minor=7, micro=0, releaselevel='final', serial=0)
 
-try:
-    from logging import NullHandler
-except ImportError:
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
-
-logging.getLogger(__name__).addHandler(NullHandler())
+logging.getLogger(__name__).addHandler(logging.NullHandler())
