@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2020 Rapptz
+Copyright (c) 2015-present Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -33,14 +33,12 @@ import traceback
 import aiohttp
 
 from .user import User, Profile
-from .asset import Asset
 from .invite import Invite
 from .template import Template
 from .widget import Widget
 from .guild import Guild
 from .channel import _channel_factory
 from .enums import ChannelType
-from .member import Member
 from .mentions import AllowedMentions
 from .errors import *
 from .enums import Status, VoiceRegion
@@ -282,6 +280,18 @@ class Client:
         ws = self.ws
         return float('nan') if not ws else ws.latency
 
+    def is_ws_ratelimited(self):
+        """:class:`bool`: Whether the websocket is currently rate limited.
+
+        This can be useful to know when deciding whether you should query members
+        using HTTP or via the gateway.
+
+        .. versionadded:: 1.6
+        """
+        if self.ws:
+            return self.ws.is_ratelimited()
+        return False
+
     @property
     def user(self):
         """Optional[:class:`.ClientUser`]: Represents the connected client. ``None`` if not logged in."""
@@ -485,6 +495,8 @@ class Client:
             Keyword argument that specifies if the account logging on is a bot
             token or not.
 
+            .. deprecated:: 1.7
+
         Raises
         ------
         :exc:`.LoginFailure`
@@ -499,10 +511,13 @@ class Client:
         await self.http.static_login(token.strip(), bot=bot)
         self._connection.is_bot = bot
 
+    @utils.deprecated('Client.close')
     async def logout(self):
         """|coro|
 
         Logs out of Discord and closes all connections.
+        
+        .. deprecated:: 1.7
 
         .. note::
 
@@ -663,7 +678,7 @@ class Client:
             try:
                 loop.run_until_complete(start(*args, **kwargs))
             except KeyboardInterrupt:
-                loop.run_until_complete(logout())
+                loop.run_until_complete(close())
                 # cancel all tasks lingering
             finally:
                 loop.close()
@@ -742,9 +757,7 @@ class Client:
 
     @allowed_mentions.setter
     def allowed_mentions(self, value):
-        if value is None:
-            self._connection.allowed_mentions = value
-        elif isinstance(value, AllowedMentions):
+        if value is None or isinstance(value, AllowedMentions):
             self._connection.allowed_mentions = value
         else:
             raise TypeError('allowed_mentions must be AllowedMentions not {0.__class__!r}'.format(value))
@@ -1063,9 +1076,7 @@ class Client:
     # Guild stuff
 
     def fetch_guilds(self, *, limit=100, before=None, after=None):
-        """|coro|
-
-        Retrieves an :class:`.AsyncIterator` that enables receiving your guilds.
+        """Retrieves an :class:`.AsyncIterator` that enables receiving your guilds.
 
         .. note::
 
@@ -1215,15 +1226,13 @@ class Client:
         if icon is not None:
             icon = utils._bytes_to_base64_data(icon)
 
-        if region is None:
-            region = VoiceRegion.us_west.value
-        else:
-            region = region.value
+        region = region or VoiceRegion.us_west
+        region_value = region.value
 
         if code:
-            data = await self.http.create_from_template(code, name, region, icon)
+            data = await self.http.create_from_template(code, name, region_value, icon)
         else:
-            data = await self.http.create_guild(name, region, icon)
+            data = await self.http.create_guild(name, region_value, icon)
         return Guild(data=data, state=self._connection)
 
     # Invite management
@@ -1375,11 +1384,14 @@ class Client:
         data = await self.http.get_user(user_id)
         return User(state=self._connection, data=data)
 
+    @utils.deprecated()
     async def fetch_user_profile(self, user_id):
         """|coro|
 
         Gets an arbitrary user's profile.
 
+        .. deprecated:: 1.7
+
         .. note::
 
             This can only be used by non-bot accounts.
